#Author: Kishan Ratnayake (ratki01)
#Created on: 02.14.2013
#Last Updated on: 10.13.2014
#
#Description:
#===========
#This script allows you to perform interface speed updates in bulk.
#Create a group in CAPC and populate it with interfaces by group rules. 
#Interfaces added manually to the group will not be updated.
#When running the script you will need to provide the following input in the following sequence:
#<DA_IP/Hostname>:<PORT> <PC_IP/Hostname>:<PORT> <PC_USER:PC_PASS> <GROUP NAME> <SpeedIn> <SpeedOut>
#####################################################################################################

use strict;
use warnings;
use REST::Client;
use Path::Class;
use XML::Tidy;
use autodie; # die if problem reading or writing a file

if(scalar(@ARGV) < 6)
{
        print "The required input was not specified. \n";
        print "Please provide the folloing input in the same exact sequence: \n";
        print "<DA_IP/Hostname>:<PORT> <PC_IP/Hostname>:<PORT> <PC_USER:PC_PASS> <GROUP NAME> <SpeedIn> <SpeedOut>";
        exit 1;
}

my $DA_URL = $ARGV[0];
my $PC_URL = $ARGV[1];
my $PC_creds = $ARGV[2];
my $group = $ARGV[3];
my $update_speedIn = $ARGV[4];
my $update_speedOut = $ARGV[5]; 

use Cwd 'abs_path';
#abs_path($0) =~ m/(.*\/)(.*\.pl)/;
abs_path($0) =~ m/(.*(\/|\\))(.*\.pl)/;
my $current_working_dir = $1;
if($^O =~ m/Win/)
{
	$current_working_dir =~ s/\//\\/g;
}

#print "Current Working Dir is: " . $current_working_dir . "\n";

my $client_da = REST::Client->new();
$client_da->setHost('http://'.$DA_URL);    
$client_da->addHeader('Accept', 'application/xml');

my $client_pc = REST::Client->new();
$client_pc->setHost('http://'.$PC_creds.'@'.$PC_URL);
$client_pc->addHeader('Accept', 'application/xml');
print "Processing....please wait.\n";
$client_pc->GET('/pc/center/webservice/groups/groupPath/All Groups');
my $tidy_obj = XML::Tidy->new( xml => $client_pc->responseContent());
$tidy_obj->tidy();
$tidy_obj->write($current_working_dir.'Groups.txt');
my $dir = dir($current_working_dir);
my $file = $dir->file('Groups.txt');
my $content = $file->slurp();
my $file_handle = $file->openr();
my $filter_top = '<FilterSelect xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="filter.xsd">' . "\n" .
'<Filter>' . "\n";

my $select_section_interface='</Filter>' . "\n" .
'<Select use="exclude" isa="exclude">' . "\n" .
'<Port use="exclude">' . "\n" .
'<Type use="include"/>' . "\n" .
'<SpeedIn use="include"/>' . "\n" .
'<SpeedOut use="include"/>' . "\n" .
'</Port>' . "\n" .
'<Item use="exclude">' . "\n" .
'<DisplayName use="include"/>' . "\n" .
'<DisplayDescription use="include"/>' . "\n" .
'</Item>'. "\n" .
'<DeviceComponent use="exclude">'. "\n" .
'<DeviceItemID use="include"/>'. "\n" .
'</DeviceComponent>'. "\n" .
'</Select>' . "\n" .
'</FilterSelect>';

my $select_section_device = '</Filter>' . "\n" .
'<Select use="exclude" isa="exclude">' . "\n" .
'<Device use="exclude">' . "\n" .
'<HostName use="include"/>' . "\n" .
'</Device>' . "\n" .
'<Item use="exclude">' . "\n" .
'<DisplayName use="include"/>' . "\n" .
'<DisplayDescription use="include"/>' . "\n" .
'</Item>'. "\n" .
'</Select>' . "\n" .
'</FilterSelect>'; 


my $matched = 0;
my $x=0;
my $j=0;
my $k=0;
my $valueList=0;
my $interfaceset=0;
my $deviceset=0;
my $type='';
my $operator='';
my $prop_name = '';
my $filter_section='';
my @interface_compare_array;
my @device_compare_array;
print "Processing....please be patient.\n";
while( my $line = $file_handle->getline() ) {
   
           $line =~ s/^\s+//; 
           $line =~ s/\s+$//;
           
            if($line =~ m/\<Group .* name="$group" .*>/)
            {
                       $matched = 1;
                       #print "GROUP FOUND: " . $group . "\n";
            }
           
            if(($matched == 1) && ($line =~ m/\<Rule add="(.*)" itemTypeLabel=(.*)\>/))
            {
              #print "Add Rule: " . $1 . "\n";
              if($1 ne 'interface')
              {
                print "The 'Add' option for the group rule is not set to 'Interfaces'.\nPlease set the 'Add' option to 'Interfaces' for the group rule and try again.";
                exit 1;
              }
            }

            if(($matched == 1) && ($line =~ m/\<Compare.* using="(.*)"\>/))
            {
                 if($x > 0)
                 {
                             $operator = $1;
                             #print $operator . "\n";
                             if($operator !~ m/^(STARTS_WITH|LIKE|EQUALS|ENDS_WITH)/)
                             {
                               print "An unsupported operator was used in the rule definition.\nSupported operators are: \n";
                               print "STARTS_WITH|LIKE|EQUALS|ENDS_WITH";
                               exit 1;
                             }
                             
                             if($operator eq 'LIKE')
                             {
                                $operator = 'CONTAINS';
                             }
                             
                             if($operator eq 'EQUALS')
                             {
                                $operator = 'EQUAL';
                             }
                             
                             #print "Operator: " . $operator . "\n";
                          
                 }
                 $x++;       
            }
            
            if(($matched == 1) && ($line =~ m/<Property label="(.*)" name="(.*)" type="(.*)" typeLabel="(.*)" typeLabels="(.*)" \/\>/) && ($x > 1))
            {
                #print "2nd Property found for group...\n";
                $prop_name = $2;
                my $temp_type = $3;
                if($temp_type !~ m/^(interface|device)$/)
                {
                  print "An unsupported Type was used in the rule definition. \nSupported Types are:\n";
                  print "Interfaces|Devices";
                  exit 1;
                }
                
                if($prop_name !~ m/^(IfAlias|ItemName|DescriptionOverride|DisplayName)$/)
                {
                  print "An unsupported Property was used in the rule definition. \n";
                  print "Supported Properties\n";
                  print "---------------------\n";
                  print "For Type 'Interfaces':\n";
                  print "Interface Name|Interface Alias|Interface Description\n\n";
                  print "For Type 'Devices':\n";
                  print "Device Name\n";
                  
                  exit 1;
                }
                
                if($temp_type =~ m/interface/)
                {
                  $type=1;
                  $interfaceset=1;
                }
                if($temp_type =~ m/device/)
                {
                  $type=2;
                  $deviceset=1;
                }
                            
            }
            
            if(($matched == 1) && ($line =~ m/\<ValueList\>/) && ($x > 1))
            {
                $valueList = 1;
            }
            
            if(($matched == 1) && ($line =~ m/\<Value\>(.*)\<\/Value\>/) && ($x > 1))
            {
              my $value = $1;
              
              if($valueList == 1)
              {
                
                    if($type == 1)
                    {
                        if($prop_name =~ m/IfAlias/)
                        {
                          $filter_section = $filter_section . '<Port.Alias type="'.$operator.'">'.$value.'</Port.Alias>'. "\n";
                        }
                      
                        #UI shows Interface Name 
                        if($prop_name =~ m/ItemName/)
                        {
                          #$select_section = '<Item.Name>';
                          $filter_section = $filter_section . '<Item.Name type="'.$operator.'">'.$value.'</Item.Name>'. "\n";
                        }
                      
                        #UI shows Interface Description
                        if($prop_name =~ m/DescriptionOverride/)
                        {
                          #$select_section = '<Item.Description>';
                          $filter_section = $filter_section . '<Item.Description type="'.$operator.'">'.$value.'</Item.Description>'. "\n";
                        }  
                    }
                    else
                    {
                        #UI shows Device Name
                        if($prop_name =~ m/DisplayName/)
                        {
                          $filter_section = $filter_section . '<Item.DisplayName type="'.$operator.'">'.$value.'</Item.DisplayName>'. "\n";
                        }
                    } 
              }
              else
              {
                    if($type == 1)
                    {
                        if($prop_name =~ m/IfAlias/)
                        {
                          $filter_section = '<Port.Alias type="'.$operator.'">'.$value.'</Port.Alias>';
                        }
                      
                        #UI shows Interface Name 
                        if($prop_name =~ m/ItemName/)
                        {
                          #$select_section = '<Item.Name>';
                          $filter_section = '<Item.Name type="'.$operator.'">'.$value.'</Item.Name>';
                        }
                      
                        #UI shows Interface Description
                        if($prop_name =~ m/DescriptionOverride/)
                        {
                          #$select_section = '<Item.Description>';
                          $filter_section = '<Item.Description type="'.$operator.'">'.$value.'</Item.Description>';
                        }
                    }
                    else
                    {
                        if($prop_name =~ m/DisplayName/)
                        {
                          $filter_section = '<Item.DisplayName type="'.$operator.'">'.$value.'</Item.DisplayName>';
                        }
                    }
              }
              
            }
            
            if(($matched == 1) && ($line =~ m/\<\/Compare\>/) && ($x > 1))
            {
                if($valueList == 1)
                {
                  if($type == 1)
                  {
                    
                    $interface_compare_array[$j++] = '<Or>' . $filter_section . '</Or>';
                  }
                  else
                  {
                    
                    $device_compare_array[$k++] = '<Or>' . $filter_section . '</Or>';
                  }
                   
                }
                else
                {
                  if($type ==1)
                  {
                    $interface_compare_array[$j++] = $filter_section;
                  }
                  else
                  {
                    $device_compare_array[$k++] = $filter_section;
                  }
                  
                }
                $valueList = 0;
                $filter_section='';
                
            }
            
            if(($matched == 1) && ($line =~ m/\<\/Group\>/))
            {
               $matched = 0;
               $x = 0;
            }
}

if($interfaceset == 0)
{
  print "A condition to filter interfaces has not been defined for the group. Please add a condition for interfaces for this group.";
  exit 1;  
}

$filter_section = '';
my $interface_filter_section='';
my $device_filter_section='';


for($x=0;$x<scalar(@interface_compare_array);$x++)
{
    if($interface_compare_array[$x] !~ m/^\<Or>/)
    {
        $filter_section = $filter_section . $interface_compare_array[$x] . "\n";
        $interface_compare_array[$x] = 'DONE';
    }
    
}
for($x=0;$x<scalar(@interface_compare_array);$x++)
{
    if($interface_compare_array[$x] !~ m/^DONE/)
    {
        $filter_section = $filter_section . $interface_compare_array[$x] . "\n";
    }
    
}

$interface_filter_section = '<And>' . "\n" . '<Pollable.IsPolled type="CONTAINS">true</Pollable.IsPolled>' . "\n" . $filter_section .'</And>' . "\n";

my $final_interface_xml = $filter_top .  $interface_filter_section  . $select_section_interface;
#print "=======================================\n";
#print $final_interface_xml;
#print "=======================================\n";
print "Processing....please wait.\n";
$client_da->POST('/rest/ports/filtered/', $final_interface_xml);
#print "<Responsecode>".$client_da->responseCode()."</Responsecode>\n";
#print "=========FILTERED CONTENT==============\n";
#print $client_da->responseContent()."\n";
#print "=========END FILTERED CONTENT============\n";
my $tidy_obj3 = XML::Tidy->new( xml => $client_da->responseContent());
$tidy_obj3->tidy();
$tidy_obj3->write($current_working_dir.'filtered_interfaces.txt');

$file = $dir->file('filtered_interfaces.txt');
$content = $file->slurp();
$file_handle = $file->openr();
my @filtered_int_id;
my @filtered_deviceitem_id;
my @filtered_int_name;

while( my $line = $file_handle->getline() ) 
{
  $line =~ s/^\s+//; 
  $line =~ s/\s+$//;
  
  if($line =~ m/\<ID\>(.*)\<\/ID\>/)
  {
    push(@filtered_int_id,$1);
  }
   
  if($line =~ m/\<DeviceItemID\>(.*)\<\/DeviceItemID\>/)
  {
    push(@filtered_deviceitem_id,$1);
  }
  if($line =~ m/\<DisplayName\>(.*)\<\/DisplayName\>/)
  {
    push(@filtered_int_name,$1);
  }
  
}

$filter_section = '';  
my $final_device_xml='';  
if($deviceset == 1)
{
  for($x=0;$x<scalar(@device_compare_array);$x++)
  {
      if($device_compare_array[$x] !~ m/^\<Or\>/)
      {
          $filter_section = $filter_section . $device_compare_array[$x] . "\n";
          $interface_compare_array[$x] = 'DONE';
      }
  }
  
  for($x=0;$x<scalar(@device_compare_array);$x++)
  {
      if($device_compare_array[$x] !~ m/^DONE/)
      {
          $filter_section = $filter_section . $device_compare_array[$x] . "\n";
      }
  }

  if(scalar(@device_compare_array) > 1)
  {
      $device_filter_section = '<And>' . "\n" . $filter_section .'</And>' . "\n";
  }
  else
  {
      $device_filter_section = "\n" . $filter_section . "\n";
  }


  $final_device_xml = $filter_top .  $device_filter_section  . $select_section_device;


  print "Processing....please be patient.\n";
  $client_da->POST('/rest/devices/filtered/', $final_device_xml);
  #print "<Responsecode>".$client_da->responseCode()."</Responsecode>\n";
  #print $client->responseContent()."\n";
  my $tidy_obj2 = XML::Tidy->new( xml => $client_da->responseContent());
  $tidy_obj2->tidy();
  $tidy_obj2->write($current_working_dir.'filtered_devices.txt');


  $file = $dir->file('filtered_devices.txt');
  $content = $file->slurp();
  $file_handle = $file->openr();
  my @filtered_dev_id;
  my @filtered_dev_name;
  my $in_device = 0;
  while( my $line = $file_handle->getline() ) 
  {
    $line =~ s/^\s+//; 
    $line =~ s/\s+$//;
    if($line =~ m/\<Device version="1.0.0"\>/)
    {
      $in_device = 1;
    }
    if(($line =~ m/\<ID\>(.*)\<\/ID\>/) && ($in_device == 1))
    {
      push(@filtered_dev_id,$1);
      $in_device = 0;
    }
    if($line =~ m/\<DisplayName\>(.*)\<\/DisplayName\>/)
    {
      push(@filtered_dev_name,$1);
    }
  }

  my $a = 0;
  $j=0;
  my @interface_ids_to_update;
  my @interfaces_names_to_update;
  for($x=0;$x<scalar(@filtered_dev_id);$x++)
  {
    for($a=0;$a<scalar(@filtered_deviceitem_id);$a++)
    {
      if($filtered_dev_id[$x] eq $filtered_deviceitem_id[$a])
      {
              $interface_ids_to_update[$j++] = $filtered_int_id[$a];
              print "Interface " . $filtered_int_name[$a] . " will be updated. \n";
              #print "ID of interface is " . $filtered_int_id[$a] . "\n";
              print "Name of interface is " . $filtered_int_name[$a] . "\n";
              print "Device that has this interface: " . $filtered_dev_name[$x] . "\n";
              print "++++++++++++++++++++++++++++++++++++++++++++++++++\n";
      }
    }
    
  }

  print "Total Number of Interfaces that need updating: " . scalar(@interface_ids_to_update) . "\n";


  my $interface_update_xml = '<Port version="1.0.0">' ."\n".
                             '<SpeedInOverride>'.$update_speedIn.'</SpeedInOverride>'."\n" .
                             '<SpeedOutOverride>'.$update_speedOut.'</SpeedOutOverride>'."\n".
                             '</Port>';
  my $success=0;
  my $nosuccess=0;
  for($x=0;$x<scalar(@interface_ids_to_update);$x++)
  {
    $client_da->PUT('/rest/ports/'.$interface_ids_to_update[$x], $interface_update_xml);
    #print "<Responsecode>".$client_da->responseCode()."</Responsecode>\n";
    if($client_da->responseCode() eq '200')
    {
      print "Successfully updated interface " . $interface_ids_to_update[$x] . "\n";
      $success++;
    }
    else
    {
      print "Unsuccessful updating interface " . $interface_ids_to_update[$x] . "\n";
      $nosuccess++;
    }
  }
  print "\n\nTotal number of interfaces that needed updating: " . scalar(@interface_ids_to_update) . "\n";
  print "Total number of interfaces successfully updated:" . $success . "\n";
  print "Total number of interfaces not updated:" . $nosuccess . "\n";
}
else
{
  print "Total Number of Interfaces that need updating: " . scalar(@filtered_int_id) . "\n";
  my $interface_update_xml = '<Port version="1.0.0">' ."\n".
                             '<SpeedInOverride>'.$update_speedIn.'</SpeedInOverride>'."\n" .
                             '<SpeedOutOverride>'.$update_speedOut.'</SpeedOutOverride>'."\n".
                             '</Port>';
  my $success=0;
  my $nosuccess=0;
  for($x=0;$x<scalar(@filtered_int_id);$x++)
  {
    $client_da->PUT('/rest/ports/'.$filtered_int_id[$x], $interface_update_xml);
    if($client_da->responseCode() eq '200')
    {
      print "Successfully updated interface " . $filtered_int_id[$x]. "\n";
      print "Interface name: " .  $filtered_int_name[$x] . "\n";
      print "++++++++++++++++++++++++++++++++++++++++++++++++++\n";
      $success++;
    }
    else
    {
      print "Unsuccessful updating interface " . $filtered_int_id[$x]. "\n";
      print "Interface name: " .  $filtered_int_name[$x] . "\n";
      print "--------------------------------------------------\n";
      $nosuccess++;
    }
  }
  print "\n\nTotal number of interfaces that needed updating: " . scalar(@filtered_int_id) . "\n";
  print "Total number of interfaces successfully updated:" . $success . "\n";
  print "Total number of interfaces not updated:" . $nosuccess . "\n";
}